###############################################################################
# Compute the statistic trace(W1 * W2^(-1)) or det(W1 * W2^(-1))
###############################################################################

wishart.ratio <- function( N=100, r=2, K=10, method="determinant" )
{
	imethod <- charmatch(method, c("trace","determinant"),
		nomatch=0)
	switch(imethod+1, 
	stop("method is unknown"),
	{
		w <- complex(N)
		for (i in 1:N)
		{
			d <- matrix(complex(real=rnorm(r*K, 0, 1/sqrt(2)), 
				im=rnorm(r*K, 0, 1/sqrt(2))), 
				nrow=r, ncol=K)
			a <- d %*% t(Conj(d)) 

			d <- matrix(complex(real=rnorm(r*K, 0, 1/sqrt(2)), 
				im=rnorm(r*K, 0, 1/sqrt(2))), 
				nrow=r, ncol=K)
			b <- d %*% t(Conj(d)) 

			w[i] <- sum(diag(a %*% solve(b)))
		}
	}
	,
	{
		w <- complex(N)
		for (i in 1:N)
		{
			d <- matrix(complex(real=rnorm(r*K, 0, 1/sqrt(2)), 
				im=rnorm(r*K, 0, 1/sqrt(2))), 
				nrow=r, ncol=K)
			a <- d %*% t(Conj(d)) 

			d <- matrix(complex(real=rnorm(r*K, 0, 1/sqrt(2)), 
				im=rnorm(r*K, 0, 1/sqrt(2))), 
				nrow=r, ncol=K)
			b <- d %*% t(Conj(d)) 

			w[i] <- det(a %*% solve(b))
		}
	}
	)
	Re(w)
}

##############################################################################
# Compute the determinant of a square matrix
# The code is copied from p. 54 of Phil Spector's S book
##############################################################################

det <- function( x )
{
	prod(diag(qr(x)$qr)) * ifelse(nrow(x) %% 2, 1, -1)
}

##############################################################################
# Compare the distributions of two samples by qqplot
##############################################################################

myqq <- function( sample1, sample2, xlab=deparse(substitute(sample1)),
				    ylab=deparse(substitute(sample2)) )
{
	x <- quantile(sample1, seq(0.05, 0.95, by=0.05))
	y <- quantile(sample2, seq(0.05, 0.95, by=0.05))
	qqplot(x, y, xlab=xlab, ylab=ylab)
	abline(0, 1)
	invisible()
}

##############################################################################
# Chi-square two-sample test (It Does NOT work!  ;^b )
##############################################################################

chi2 <- function( sample1, sample2, bin=5 )
{
#	breaks <- seq(min(sample2), max(sample2), length=(bin+1))
#	breaks[1] <- -Inf
#	breaks[bin+1] <- Inf

	inner <- seq(quantile(sample2,.05),quantile(sample2,.95),length=bin-1)
	breaks <- c(-Inf, inner, Inf)

	p <- vector("numeric", length=bin)
	n <- vector("numeric", length=bin)
	for (i in 1:bin)
	{
		p[i] <- sum(breaks[i] < sample2 & sample2 <= breaks[i+1])
		n[i] <- sum(breaks[i] < sample1 & sample1 <= breaks[i+1])
	}

print(round(p/length(sample2), 3))
print(round(n/length(sample1), 3))

	p <- length(sample1) * p / length(sample2)
	chi <- sum((n - p)^2/p)

#	cat("chi2 =", round(sum((n - p)^2/p),3), "\n")
	cat("pvalue =", round(1 - pchisq(chi, bin-1), 6), "\n")
#	cat("critical =", round(qchisq(.95, bin-1), 3), "\n")
}

##############################################################################
# Kolmogorov-Smirnov two-sample test.  It returns the p-value.  
##############################################################################

kstwo <- function( sample1, sample2 )
{
        if (!is.loaded(C.symbol("call_ks"))) dyn.load("call_ks.o")
#	dyn.load("call_ks.o")
	z <- .C("call_ks", 
		as.single(sort(sample1)),
		as.single(sort(sample2)),
		as.integer(length(sample1)),
		as.integer(length(sample2)),
		d=as.single(numeric(1)),
		p=as.single(numeric(1)))
#cat("ks-stat =", round(z$d, 6), "\n")
#cat("p-value =", round(z$p, 6), "\n")
#	invisible()
	z$p	
}


